WhisperQ = LibStub("AceAddon-3.0"):NewAddon("WhisperQ", "AceEvent-3.0");
local ldb = LibStub:GetLibrary("LibDataBroker-1.1");

local LibQTip = LibStub("LibQTip-1.0");
local tooltip;

local Dewdrop = AceLibrary("Dewdrop-2.0");

local WQicon = "Interface\\Icons\\Spell_Shadow_PsychicScream"

WhisperQ.WQ = {};
local WQ = WhisperQ.WQ
WQ.DEBUG_MODE = false
WQ.RealmName = nil;

local defaults = {
	profile = {
		MessageLimit = 15,
		PlayerLimit = 12,
		ColorIn = { .38, .88, .98 };
		ColorOut = { .55, 1, 0.2 };
	},
	char = {
		Whispers = {},
	},
}

function WQ.Debug(...)
	if(WQ.DEBUG_MODE) then
		local output, part
		for i=1, select("#", ...) do
			part = select(i, ...)
			part = tostring(part):gsub("{{", "|cffddeeff"):gsub("}}", "|r")
			if (output) then output = output .. " " .. part
			else output = part end
		end
		ChatFrame1:AddMessage(output, 1.0, 1.0, 0.7);
	end
end
			
function WQ.AddWhisper(Whispers, author, message, from)
	if string.sub(message,1,string.len("OQ")) == "OQ" then
		return;
	end
	local msg = {message, time(), from}

	for i, j in pairs(Whispers) do
		if j[1] == author then
			if i ~= 1 then
				table.remove(Whispers, i);
				table.insert(Whispers, 1, j);
			end
			table.insert(j[2], 1, msg);
			while #(j[2]) > WQ.db.profile.MessageLimit do
				table.remove(j[2], WQ.db.profile.MessageLimit+1);
			end
			while #(Whispers) > WQ.db.profile.PlayerLimit do
				table.remove(Whispers, WQ.db.profile.PlayerLimit+1);
			end
			WQ.broker.text = author;
			return;
		end
	end
	local msgtable = {}
	table.insert(msgtable, msg);
	local data = { author, msgtable }
	table.insert(Whispers, 1, data);

	while #(Whispers) > WQ.db.profile.PlayerLimit do
		table.remove(Whispers, WQ.db.profile.PlayerLimit+1);
	end
	WQ.broker.text = author;
end

function WhisperQ:CheckRealm()
	if WQ.RealmName == nil then
		WQ.RealmName = GetRealmName();
		if WQ.RealmName ~= nil then
			WQ.RealmName = string.gsub(WQ.RealmName, "%s+", "");
		end
	end
end

function WhisperQ:OnReceiveWhisper(event, message, author)
	if (author:find("^|K")) then
		local presenceID = string.match(author, "(%d+)");
		local _,_,_,_,toonName, toonID = BNGetFriendInfoByID(presenceID);
		local _,_,_,realm,realmID = BNGetToonInfo(toonID)
		if realm ~= nil then
			realm = string.gsub(realm, "%s+", "")
		end
		if realm ~= nil and realm ~= "" and realm ~= WQ.RealmName then
			author = toonName.."-"..realm;
		elseif realm ~= WQ.RealmName then
			author = toonName.." (BNet)";
		else
			author = toonName;
		end
	else
		local name, realm = strsplit("-", author);
		if realm == WQ.RealmName then
			author = name;
		end
	end
	local Whispers = WQ.db.char.Whispers;
	WQ.AddWhisper(Whispers, author, message, 1);
end

function WhisperQ:OnSendWhisper(event, message, author)
	if (author:find("^|K")) then
		local presenceID = string.match(author, "(%d+)");
		local _,_,_,_,toonName, toonID = BNGetFriendInfoByID(presenceID);
		if toonID == nil then 
			author = "Unknown";
		else
			local _,_,_,realm = BNGetToonInfo(toonID)
			if realm ~= nil then
				realm = string.gsub(realm, "%s+", "")
			end
			if realm ~= nil and realm ~= "" and realm ~= WQ.RealmName then
				author = toonName.."-"..realm;
			elseif realm ~= WQ.RealmName then
				author = toonName.." (BNet)";
			else
				author = toonName;
			end
		end
	else
		local name, realm = strsplit("-", author);
		if realm == WQ.RealmName then
			author = name;
		end
	end
	local Whispers = WQ.db.char.Whispers;
	WQ.AddWhisper(Whispers, author, message, 0);
end

function WhisperQ:OnInitialize()
	WQ.db = LibStub("AceDB-3.0"):New("WhisperQDB", defaults, true);
--	WQ.db:ResetDB();

	WQ.broker = ldb:NewDataObject("WhisperQ", {
		type = "data source", 
		icon = WQicon, 
		label = "WhisperQ",
		text = "",
		OnClick = function(self, button) WhisperQ:OnClick(self, button) end,
		OnEnter = function(self, button) WhisperQ:OnEnter(self, button) end,
		OnLeave = function(self, button) WhisperQ:OnLeave(self, button) end,

	});
end

function WhisperQ:OnEnable()
	self:RegisterEvent("CHAT_MSG_WHISPER", "OnReceiveWhisper")
	self:RegisterEvent("CHAT_MSG_WHISPER_INFORM", "OnSendWhisper")
	self:RegisterEvent("CHAT_MSG_BN_WHISPER", "OnReceiveWhisper");
	self:RegisterEvent("CHAT_MSG_BN_WHISPER_INFORM", "OnSendWhisper");

	local LastWhisper = WQ.db.char.Whispers[1];
	if LastWhisper ~= nil then
		local LastName = LastWhisper[1];
		if LastName ~= nil then
			WQ.broker.text = LastName;
		end
	end
	WhisperQ:CheckRealm();
end

function WhisperQ:ColorText(s, colors)
	return "|c"..string.format("%02x%02x%02x%02x", 255, colors[1]*255, colors[2]*255, colors[3]*255)..s.."|r";
end

local WQ_Config;
function WQ.BuildNamesList()
	WQ_Config = 1;
	for i, j in pairs(WQ.db.char.Whispers) do
		Dewdrop:AddLine("text", j[1], 'tooltipText', "", "hasArrow", true, "value", i);
		WQ_Config = i + 1;
	end

	Dewdrop:AddLine("text", "", "isTitle", true);
	Dewdrop:AddLine("text", "|cffffff00Config|r", 'tooltipText', "", "hasArrow", true, "value", WQ_Config);
end

function WhisperQ:OnClick(self, button)
	if button == "RightButton" then
		if tooltip then
			LibQTip:Release(tooltip);
			tooltip = nil;
		end
		Dewdrop:Register(self, "children", function(level, value, arg1, arg2, arg3, arg4)
			if level == 1 then 
				Dewdrop:AddLine("text", "|cff00E5EEWhisperQ|r", "isTitle", true);
				WQ.BuildNamesList();
			end
			if level == 2 then
				if value == WQ_Config then
					Dewdrop:AddLine("text", "Max Players", 'hasArrow', true, 'hasSlider', true, 'sliderMin', 1, 'sliderMax', 50, 'sliderStep', 1.0, 'sliderValue', WQ.db.profile.PlayerLimit, 'sliderFunc', function(value) WQ.db.profile.PlayerLimit = value; end, 'tooltipText', "");
					Dewdrop:AddLine("text", "Whispers Saved", 'hasArrow', true, 'hasSlider', true, 'sliderMin', 1, 'sliderMax', 50, 'sliderStep', 1.0, 'sliderValue', WQ.db.profile.MessageLimit, 'sliderFunc', function(value) WQ.db.profile.MessageLimit = value; end, 'tooltipText', "");
					Dewdrop:AddLine('text', "Incoming Color", 'hasColorSwatch', true, 'hasOpacity', true, 'r', WQ.db.profile.ColorIn[1], 'g', WQ.db.profile.ColorIn[2], 'b', WQ.db.profile.ColorIn[3], 'colorFunc', function(r, g, b, a) WQ.db.profile.ColorIn = { r, g, b }; end, 'tooltipText', "");
					Dewdrop:AddLine('text', "Outgoing Color", 'hasColorSwatch', true, 'hasOpacity', true, 'r', WQ.db.profile.ColorOut[1], 'g', WQ.db.profile.ColorOut[2], 'b', WQ.db.profile.ColorOut[3], 'colorFunc', function(r, g, b, a) WQ.db.profile.ColorOut = { r, g, b }; end, 'tooltipText', "");
				else
					WQ.Debug("Value = "..value);
					local person = WQ.db.char.Whispers[value];
					WQ.Debug("Person is "..person[1]);
					for i, j in pairs(person[2]) do
						WQ.Debug("Message is "..j[1]);
						local lines = 1;
						local msgs = WQ.SplitString(j[1], false);
						for a,b in pairs(msgs) do
							local addtime = "";
							local colortext;
							if lines == 1 then
								addtime = date("%m/%d %H:%M", tonumber(j[2])).." ";
							end
							if j[3] == 1 then
								colortext = WhisperQ:ColorText(b, WQ.db.profile.ColorIn);
							else
								colortext = WhisperQ:ColorText(b, WQ.db.profile.ColorOut);
							end
							Dewdrop:AddLine("text", addtime..colortext, "notCheckable", true, "justifyH", "LEFT", 'tooltipText', "");
							lines = lines + 1;
						end
					end
				end
			end
		end);
	Dewdrop:Open(self, "cursorX", true, "cursorY", true);
end
	if button == "LeftButton" and IsShiftKeyDown() and #WQ.db.char.Whispers > 0 then
		table.remove(WQ.db.char.Whispers, 1);
		if #WQ.db.char.Whispers > 0 then
			for i,j in pairs(WQ.db.char.Whispers) do
				WQ.broker.text = j[1];
				break;
			end
		else
			WQ.broker.text = "";
		end
		WhisperQ:OnEnter(self)
	end
end

function WhisperQ:OnEnter(self, button)
	tooltip = LibQTip:Acquire("WisperQTooltip", 2, "LEFT", "LEFT");
	tooltip:SetScale(1);
	tooltip:Clear();

	local line = tooltip:AddLine();
	tooltip:SetCell(line, 1, "|cff00E5EEWhisperQ|r", nil, "CENTER", 2);
	tooltip:AddSeparator();

	WQ.UpdateTooltip(tooltip, WQ.db.char.Whispers[1]);

	tooltip:EnableMouse()
	tooltip:SmartAnchorTo(self)
	tooltip:SetAutoHideDelay(0.25, self)
	tooltip:UpdateScrolling()
	tooltip:Show()
end

function WhisperQ:OnLeave(self, button)
	if tooltip then
		LibQTip:Release(tooltip);
		tooltip = nil;
	end
end

function WQ.UpdateTooltip(tooltip, Whispers)
	if Whispers == nil then
		local line = tooltip:AddLine();
		tooltip:SetCell(line, 1, "Welcome to WhisperQ!");
		line = tooltip:AddLine();
		tooltip:SetCell(line, 1, "You have no current whispers.");
		line = tooltip:AddLine();
		tooltip:SetCell(line, 1, "Right click for options.");
		line = tooltip:AddLine();
		tooltip:SetCell(line, 1, "Shift Left click to remove current whisper list.");
		return
	end
	msgs = Whispers[2];
	for k, l in pairs(msgs) do
		local format = "%s";
		local line = tooltip:AddLine();
		local display_msg, num = WQ.SplitString(l[1], true);
		local colortext;
		if l[3] == 1 then
			colortext = WhisperQ:ColorText(display_msg, WQ.db.profile.ColorIn);
		else
			colortext = WhisperQ:ColorText(display_msg, WQ.db.profile.ColorOut);
		end
		local timestamp = date("%m/%d %H:%M", tonumber(l[2]));
		tooltip:SetCell(line, 1, timestamp);
		tooltip:SetCell(line, 2, colortext, nil, "LEFT", 1);
	end
end

function WQ.SplitString(speech, single)
	local lines = {}
	local line = {}
	local width = 0

	for w in string.gmatch(speech, "%S+") do
		width = width + string.len(w)
		table.insert(line, w) 
		if(width > 30) then
			width = 0
			table.insert(lines, table.concat(line, " "))
			line = {}
		end
	end
	if(#line > 0) then
		table.insert(lines, table.concat(line, " "))
	end
	if single then 
		return table.concat(lines, "\n"), #lines
	else
		return lines;
	end
end
